import { Dispatch, SetStateAction, createContext, useState } from "react";
import styles from "./Calendar.module.scss";
import ToggleSwitch from "src/widgets/ToggleSwitch";
import InputBox from "src/widgets/InputBox";
import Button from "src/widgets/Button";
import MonthCalendar from "./MonthCalendar";
import WeekCalendar from "./WeekCalendar";
import { usePersistentState } from "lib/persistentState";

export enum CalendarMode {
  Week = "Week",
  Month = "Month",
  Year = "Year",
}

export enum CalendarView {
  Calendar = "Calendar",
  Graph = "Graph",
}

interface ICalendarContext {
  onDateClick: (_: Date) => void;
  setCurrentTitle: Dispatch<SetStateAction<JSX.Element>>;

  viewportWidth: number;
  viewportHeight: number;
  setViewportSize: Dispatch<SetStateAction<[number, number]>>;
}

export const CalendarContext = createContext<ICalendarContext>(null!);

interface CalendarProps {
  onDateClick: (_: Date) => void;
}

export default function Calendar({ onDateClick }: CalendarProps) {
  const [currentTitle, setCurrentTitle] = useState<JSX.Element>(<></>);

  const [searchQuery, setSearchQuery] = useState("");

  const [currentMode, setCurrentMode] = usePersistentState(
    "CalendarMode",
    CalendarMode.Month
  );
  const [currentView, setCurrentView] = usePersistentState(
    "CalendarView",
    CalendarView.Calendar
  );

  const [viewportSize, setViewportSize] = useState<[number, number]>([0, 0]);
  const [viewportWidth, viewportHeight] = viewportSize;

  const contextValue = {
    onDateClick,
    setCurrentTitle,
    viewportWidth,
    viewportHeight,
    setViewportSize,
  };

  const scrollToToday = () => {};

  return (
    <CalendarContext.Provider value={contextValue}>
      <div className={styles.calendar}>
        <div className={styles.header}>
          <div className={styles.title}>
            <div>{currentTitle}</div>

            <div>
              <InputBox
                value={searchQuery}
                setValue={setSearchQuery}
                placeholder="Search..."
              />
            </div>

            <div className={styles.controlButtons}>
              <Button>Today</Button>

              <ToggleSwitch<CalendarMode>
                options={Object.values(CalendarMode) as CalendarMode[]}
                value={currentMode}
                setValue={(x) => setCurrentMode(x)}
                renderValue={(x) => <>{CalendarMode[x]}</>}
              />

              <ToggleSwitch<CalendarView>
                options={Object.values(CalendarView) as CalendarView[]}
                value={currentView}
                setValue={setCurrentView}
                renderValue={(x) => <>{CalendarView[x]}</>}
              />
            </div>
          </div>
        </div>

        <ActualCalendar mode={currentMode} />
      </div>
    </CalendarContext.Provider>
  );
}

interface ActualCalendarProps {
  mode: CalendarMode;
}

function ActualCalendar({ mode }: ActualCalendarProps) {
  switch (mode) {
    case CalendarMode.Month:
      return <MonthCalendar />;
    case CalendarMode.Week:
      return <WeekCalendar />;
    case CalendarMode.Year:
      return <MonthCalendar />;
  }
}
