import { ForwardedRef, forwardRef } from "react";
import styles from "./Month.module.scss";
import {
  addDays,
  differenceInWeeks,
  endOfMonth,
  getWeek,
  isSameDay,
  startOfMonth,
  startOfWeek,
} from "date-fns";
import classNames from "classnames";
import { weekBoundsOfMonth } from "lib/date";
import { journalFmt, useJournals } from "lib/queries";

interface MonthProps {
  month: Date;
  isActive: boolean;
  dateGridHeight: number;
  dateCellHeight: number;
  onDateClick?: (_: Date) => void;
}

function Month(
  { month, dateGridHeight, isActive, dateCellHeight, onDateClick }: MonthProps,
  ref: ForwardedRef<HTMLDivElement>
) {
  const now = new Date();
  const monthName = month.toLocaleString("default", { month: "long" });

  const firstDay = startOfMonth(month);
  const lastDay = endOfMonth(month);
  const { status, data: journals } = useJournals(firstDay, lastDay);

  const [startWeek, endWeek] = weekBoundsOfMonth(month);
  const numWeeks = differenceInWeeks(endWeek, startWeek) + 1;
  const weeksArr = Array(numWeeks)
    .fill(0)
    .map((_, i) => addDays(startOfMonth(month), 7 * i));
  const datesArr = weeksArr.flatMap((week) => {
    const start = startOfWeek(week);
    return Array(7)
      .fill(0)
      .map((_, i) => addDays(start, i));
  });

  return (
    <>
      <div
        className={classNames(styles.dateGrid, isActive && styles.active)}
        // style={{ height: `${dateGridHeight}px` }}
        style={{ gridTemplateRows: `repeat(${numWeeks}, ${dateCellHeight}px)` }}
        ref={ref}
        data-isodate={month}
      >
        {datesArr.map((date) => {
          const isFirst = isSameDay(date, startOfMonth(date));
          const isToday = isSameDay(date, now);

          const journalDate = journalFmt(date);
          const hasJournal = journals?.has(journalDate);

          return (
            <div
              key={date.toString()}
              className={styles.dateCell}
              data-isodate={date.toISOString()}
            >
              <div
                className={styles.dateNumber}
                onClick={() => onDateClick?.(date)}
              >
                <span className={classNames(isToday && styles.today)}>
                  {isFirst && monthName}
                  {date.getDate()}
                </span>
              </div>

              <div className={styles.dateCellBody}>
                <p>{hasJournal && "journal"}</p>
              </div>
            </div>
          );
        })}
      </div>
    </>
  );
}

export default forwardRef(Month);
